% Parameter estimation for the Cox, Ingersoll, Ross model.
% Uses exact form for the likelihood function
% CIR model is dr = alpha(mu - r)dt + sigma*sqrt(r)*W
%   alpha = mean reversion speed,
%   mu    = mean reversion level,
%   sigma = volatility.
% Estimate the parameters, and construct the yield curve R(t,T).
% Simulate paths for the short rate.
% Proxy the short rate using the 3-month T-bill yield as proxy.

clc; clear;

% Input the 3-month T-bill yield.
[r date] = xlsread('CIR_short_rate_data.xls', 'Data', 'A3:B2754');
r = r./100;

% Number of observations, observations are daily.
N = length(r);
dt = 1/360;

% Find the OLS estimates for alpha, mu, and sigma.
% These estimates are used as starting values for the exact likelihood.
% From "Maximum Likelihood Estimation of the CIR Process: The Matlab
% Implementation" by Kamil Kladivko.
y = (r(2:N) - r(1:N-1))./sqrt(r(1:N-1));
x1 = dt./sqrt(r(1:N-1));
x2 = sqrt(r(1:N-1)).*dt;
b = regress(y, [x1 x2]);
alpha = -b(2);
mu = b(1)/alpha;
res = y - b(1).*x1 - b(2).*x2;
sigma = std(res)./sqrt(dt);

% Estimate alpha, mu, and sigma using the exact likelihood.
% From "Theory of the Term Structure of Interest Rates" by
% Cox, Ingersoll, and Ross.  Econometrica 1985, 53(3).
start = [alpha mu sigma]
params = fminsearch(@(b) CIR_LL(b,r,dt), start)
alpha  = params(1);
mu     = params(2);
sigma  = params(3);

% Generate the yield curve R(t,T) based on the parameters.
% Inline function for A(t,T).
A = inline('(2*gam*exp((alpha+gam)*(T-t)/2)/(2*gam + (alpha + gam)*(exp(gam*(T-t))-1)))^(2*alpha*mu/sigma^2)',...
	'alpha','mu','sigma','gam','t','T');

% Inline function for B(t,T).
B = inline('2*(exp(gam*(T-t))-1) / (2*gam + (alpha+gam)*(exp(gam*(T-t))-1))',...
	'alpha','mu','gam','t','T');

% Gamma parameter.
gam = sqrt(alpha^2 + 2*sigma^2);

% Define the required settings for the yield curve.
LastTenor = N/252;        % Last tenor is about 10 years.
t = 0;                    % Time zero t=0.
Inc = 1/2;                % Increment for the yield curve tenor points.
T = [Inc:Inc:LastTenor];  % Tenor points.
CurrentRate = r(end);     % Current rate r(t) is most recent rate.

% Zero coupon bond: P(t,T) = A(t,T)*exp(B(t,T)*CurrentRate).
% Yield curve : R(t,T) = -log(P(t,T))/(T-t).
for i=1:length(T)
	P(i) = A(alpha,mu,sigma,gam,t,T(i))*exp(-B(alpha,mu,gam,t,T(i))*CurrentRate);
	R(i) = -log(P(i))/(T(i)-t)*100;
end

% Simulate 10 paths for the short rate.
Nsims = 10;
for k=1:Nsims;
	f(1,k) = r(1);
	for t=2:N
		f(t-1,k) = max(0,f(t-1,k));
		f(t,k) = f(t-1,k) + alpha*(mu - f(t-1,k))*dt + sigma*sqrt(f(t-1,k))*randn(1)*sqrt(dt);
	end
end

% Plot the results.
subplot(2,1,1)
plot(T,R)
legend('Estimated Yield Curve')
xlabel('Time');
ylabel('Yield');

subplot(2,1,2)
plot((1:N), r, 'k-o',(1:N), f)
legend('Original time series', 'Simulated series')
xlabel('Time')
ylabel('Simulated short rate')

